/**
 * Stinger is an HTTP Request Validation Engine
 * Copyright (C) 2006  Aspect Security, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact us at info@aspectsecurity.com or www.aspectsecurity.com
 *
 */

package org.owasp.stinger;

import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.owasp.stinger.http.MutableHttpRequest;
import org.owasp.stinger.http.MutableHttpResponse;
import org.owasp.stinger.rules.RuleSet;

public class StingerFilter implements Filter {
	
	private Stinger stinger = null;
	
	private String config = null;

	private boolean reload = false;
	
	private RuleSet ruleSet = null;
	
	private String errmsg = null;
	
	
	public void init(FilterConfig filterConfig) {
		
		/** Pull config location from Filter init parameter **/
		config = filterConfig.getInitParameter("config");
		ruleSet = new RuleSet(config);
		/** Error message to display when exceptions are thrown **/
		errmsg = filterConfig.getInitParameter("errmsg");
		/** Should we dynamically load the ruleset? **/
		reload = Boolean.valueOf(filterConfig.getInitParameter("reload"));
		/** Get the stinger instance **/
		stinger = Stinger.getInstance(ruleSet);
	}
	
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) {
		MutableHttpRequest mRequest = null;
		MutableHttpResponse mResponse = null;
		
		if(request instanceof HttpServletRequest && response instanceof HttpServletResponse) {
			mRequest = new MutableHttpRequest((HttpServletRequest)request);
			mResponse = new MutableHttpResponse((HttpServletResponse)response);
			
			initRuleSet();
			
			try {
				stinger.validate(mRequest, mResponse);
				
				doChain(mRequest, mResponse, response, chain);
			} catch (BreakChainException bce) {
				bce.printStackTrace();
			} catch(Exception e) {
				e.printStackTrace();
				
				try {
					response.getWriter().print(errmsg);
				} catch (IOException ioe) {
					ioe.printStackTrace();
				}
			}
		}
	}
	
	public void destroy() {
		
	}
	
	private synchronized void initRuleSet() {
		if(reload) {
			ruleSet = new RuleSet(config);
			Stinger.setRuleSet(ruleSet);
		}
	}
	
	private void doChain(HttpServletRequest mRequest, MutableHttpResponse mResponse, ServletResponse response, FilterChain chain) throws IOException, ServletException
	{
		OutputStream out = response.getOutputStream();
		
		chain.doFilter(mRequest, mResponse);
		
		String contentType = mResponse.getContentType();
		
		if(contentType != null && contentType.startsWith("text"))
		{
			String content = new String(mResponse.getData());
			byte[] result = content.getBytes();
			
			response.setContentLength(result.length);
			out.write(result);
		}
		else
		{
			out.write(mResponse.getData());
		}

		out.close();
	}
}
