/**
 * Stinger is an HTTP Request Validation Engine
 * Copyright (C) 2006  Aspect Security, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact us at info@aspectsecurity.com or www.aspectsecurity.com
 *
 */

package org.owasp.stinger.actions;

import java.io.IOException;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.logging.FileHandler;
import java.util.HashMap;

import javax.servlet.http.HttpServletResponse;

import org.owasp.stinger.http.MutableHttpRequest;
import org.owasp.stinger.violation.Violation;

public class Log extends AbstractAction {
	
	private static Logger logger = Logger.getLogger("org.owasp.stinger.actions.Log");
	
	private static HashMap<String, FileHandler> handlers = new HashMap<String, FileHandler>();
	
	public Log() {
		
	}
	
	public void doAction(Violation violation, MutableHttpRequest request, HttpServletResponse response) {
		FileHandler handler = null;
		String log = getParameter("log");
		String level = getParameter("level");
		String message = getParameter("message");
		
		/** Offender's IP **/
		message = message.replace("%ip", request.getRemoteAddr());
		/** Offender's Port **/
		message = message.replace("%port", String.valueOf(request.getRemotePort()));
		/** Offending parameter name **/
		if(violation.getName() != null) {
			message = message.replace("%name", violation.getName());
		} else {
			message = message.replace("%name", "NULL");
		}
		/** Offending parameter value **/
		if(violation.getValue() != null) {
			message = message.replace("%value", violation.getValue());
		} else {
			message = message.replace("%value", "NULL");
		}
		
		/** Offending parameter value HTML Encoded **/
		if(violation.getValue() != null) {
			message = message.replace("%encoded_value", violation.getValue());
		} else {
			message = message.replace("%encoded_value", "NULL");
		}
		
		/** Offender's JSESSIONID **/
		if(request.getCookie("JSESSIONID") != null) {
			message = message.replace("%js", request.getCookie("JSESSIONID").getValue());
		} else {
			message = message.replace("%js", "NULL");
		}
		
		handler = getHandler(log);
		logger.addHandler(handler);
		
		logger.log(Level.parse(level.toUpperCase()), message);
		handler.flush();
		
		logger.removeHandler(handler);
	}
	
	private synchronized FileHandler getHandler(String log) {
		FileHandler handler = null;
		
		handler = handlers.get(log);
		
		if(handler == null) {
			try {
				handler = new FileHandler(log);
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
			
			handlers.put(log, handler);
		}
		
		return handler;
	}
}
