/**
 * Stinger is an HTTP Request Validation Engine
 * Copyright (C) 2006  Aspect Security, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact us at info@aspectsecurity.com or www.aspectsecurity.com
 *
 */

package org.owasp.stinger.http;

import java.util.Collections;
import java.util.HashMap;
import java.util.Enumeration;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

public class MutableHttpRequest extends HttpServletRequestWrapper {
	
	private HashMap<String, String> headers = null;
	private HashMap<String, Cookie> cookies = null;
	private HashMap<String, String> parameters = null;
	
	public MutableHttpRequest(HttpServletRequest request) {
		super(request);
		parseRequest();
	}
	
	private void parseRequest() {
		String name = null;
		headers = new HashMap<String, String>();
		cookies = new HashMap<String, Cookie>();
		parameters = new HashMap<String, String>();
		Enumeration e = super.getHeaderNames();
		Cookie[] requestCookies = null;
		
		while(e.hasMoreElements()) {
			name = (String)e.nextElement();
			
			headers.put(name, super.getHeader(name));
		}
		
		requestCookies = super.getCookies();
		
		if(requestCookies != null) {
			for(int i=0; i<requestCookies.length; i++) {
				cookies.put(requestCookies[i].getName(), requestCookies[i]);
			}
		}
		
		e = super.getParameterNames();
		
		while(e.hasMoreElements()) {
			name = (String)e.nextElement();
			
			parameters.put(name, super.getParameter(name));
		}
	}
	
	public HashMap<String, String> getHeaders() {
		return headers;
	}
	
	public void setHeaders(HashMap<String, String> headers) {
		this.headers = headers;
	}
	
	public String getHeader(String name) {
		return headers.get(name);
	}
	
	public Enumeration getHeaderNames() {
		return Collections.enumeration(headers.keySet());
	}
	
	public Enumeration getHeaders(String name) {
		return null;
	}
	
	public int getIntHeader(String name) {
		String value = headers.get(name);
		int result = -1;
		
		if(value != null) {
			result = Integer.parseInt(headers.get(name));
		}
		
		return result;
	}
	
	public void addHeader(String name, String value) {
		if(headers.get(name) != null) {
			headers.remove(name);
		}
		
		headers.put(name, value);
	}
	
	public Cookie[] getCookies() {
		return (Cookie[])cookies.values().toArray( new Cookie[cookies.size()]);
	}
	
	public void setCookies(Cookie[] cookies) {
		clearCookies();
		
		for(int i=0; i<cookies.length; i++) {
			this.cookies.put(cookies[i].getName(), cookies[i]);
		}
	}
	
	public Cookie getCookie(String name) {
		return cookies.get(name);
	}
	
	public void addCookie(Cookie cookie) {
		if(cookies.get(cookie.getName()) != null) {
			cookies.remove(cookie.getName());
		}
		
		cookies.put(cookie.getName(), cookie);
	}
	
	public void clearCookies() {
		cookies = new HashMap<String, Cookie>();
	}
	
	public HashMap<String, String> getParameters() {
		return parameters;
	}
	
	public void setParameters(HashMap<String, String> parameters) {
		this.parameters = parameters;
	}
	
	public String getParameter(String name) {
		return parameters.get(name);
	}
	
	public void addParameter(String name, String value) {
		if(parameters.get(name) != null) {
			parameters.remove(name);
		}
		
		parameters.put(name, value);
	}
	
	public Enumeration getParameterNames() {
		return Collections.enumeration(parameters.keySet());
	}
	
	public Enumeration getParameterValues() {
		return Collections.enumeration(parameters.values());
	}
}
