/**
 * Stinger is an HTTP Request Validation Engine
 * Copyright (C) 2006  Aspect Security, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact us at info@aspectsecurity.com or www.aspectsecurity.com
 *
 */

package org.owasp.stinger.util;

import java.lang.StringBuffer;

import java.util.HashMap;

public class EntityEncoder {
	
	private static HashMap encodeMap = populateEncodeMap();
	
	private static HashMap decodeMap = populateDecodeMap();
    
    /** Creates a new instance of EntityEncoder **/
    private EntityEncoder() {
    }
    
    private static HashMap<String, String> populateDecodeMap() {
    	HashMap<String, String> result = new HashMap<String, String>();
    	
    	result.put("&#34;", "\"");
    	result.put("&#35;", "#");
    	result.put("&#37;", "%");
    	result.put("&#38;", "&");
    	result.put("&#39;", "'");
    	result.put("&#40;", "(");
    	result.put("&#41;", ")");
    	result.put("&#43;", "+");
    	result.put("&#45;", "-");
    	result.put("&#59;", ";");
    	result.put("&#60;", "<");
    	result.put("&#62;", ">");
    	
    	return result;
    }
    
    private static HashMap<Character, String> populateEncodeMap() {
    	HashMap<Character, String> result = new HashMap<Character, String>();
    	
    	result.put('\\', "&#34;");
    	result.put('#', "&#35;");
    	result.put('%', "&#37;");
    	result.put('&', "&#38;");
    	result.put('\'', "&#39;");
    	result.put('(', "&#40;");
    	result.put(')', "&#41;");
    	result.put('+', "&#43;");
    	result.put('-', "&#45;");
    	result.put(';', "&#59;");
    	result.put('<', "&#60;");
    	result.put('>', "&#62;");
    	
    	return result;
    }
    
    public static String htmlEntityEncode(String data) {
    	String encoded = null;
        StringBuffer buffer = new StringBuffer();
        char c = 0;
        char[] array = data.toCharArray();
        
        for(int i=0; i<array.length; i++) {
            c = array[i];
            encoded = (String)encodeMap.get(c);
            
            if(encoded != null) {
            	buffer.append(encoded);
            } else {
            	buffer.append(c);
            }
        }
        
        return buffer.toString();
    }
    
    public static String htmlEntityDecode(String data) {
        int len = data.length();
        char c = 0;
        String subString = null;
        String decoded = null;
        StringBuffer buffer = new StringBuffer();
        
        for(int i=0; i<len; i++) {
            c = data.charAt(i);
            
            if(c == '&' && (i+5) <= len) {
                subString = data.substring(i, i+5);
                
                decoded = (String)decodeMap.get(subString);
                
                i += 4;
                
                if(decoded != null) {
                	buffer.append(decoded);
                	decoded = null;
                } else {
                	i -= 4;
                	buffer.append(c);
                }
            } else {
                buffer.append(c);
            }
        }
        
        return buffer.toString();
    }
}
